from pprint import pprint
from sets import ImmutableSet


def combopack(modules, entry_id):
    def bundle(module_id):
        """
        Bundle a module with all of its hard dependencies, recursively.
        Browserify does this step.
    
        Parameters:
            module_id (str)
        """
        result = {module_id}
        module = modules[module_id]
        for dep_id in module['deps']:
            result.update(bundle(dep_id))
        return result
    
    # Define a state as its collection of modules.
    # The first state consists of just the router module
    # and its hard dependencies.
    
    class State(ImmutableSet):
        def __init__(self, module_ids):
            super(State, self).__init__(module_ids)
    
        @property
        def cdeps(self):
            """
            Here's a function that finds all the unsatisfied conditional
            dependencies for the current state. They are returned as a set
            of module identifiers.
            """
            cdep_ids = set()
            for module_id in self:
                state_module = modules[module_id]
                for other_module_id in state_module['cdeps']:
                    if other_module_id not in self:
                        cdep_ids.add(other_module_id)
            return cdep_ids
    
        def union(self, other):
            return State(
                super(State, self).union(other)
            )

        def difference(self, other):
            result = set(self).difference(set(other))
            return State(result)
    

    class Bundle(State):
        pass
    
    
    def get_state(module_ids):
        # Loop over modules in state. If the module has a conditional
        # dependency on another module that is NOT in the state, add
        # it to the state_cdeps.
        state = {
            'modules': set(module_ids),
            'cdeps': set()
        }
        for module_id in state['modules']:
            state_module = modules[module_id]
            for other_module_id in state_module['cdeps']:
                if other_module_id not in state['modules']:
                    state['cdeps'].add(other_module_id)
        return state
    
    def states(entry_id):
        # A state is a mapping where a key is a bundle and its
        # value is a mapping between the unsatisfied conditional
        # dependencies
        # of the bundle and the 
        _states = set()
    
        def child_states(state):
            """
            For a state, visit all of its unsatisfied conditional dependencies
            and form the bundles for those dependencies. Save the bundles in the
            global bundles array and then form an array of new states, that are
            concatenations of the module list in the given state and each bundle.
    
            Return the array of states.
            """
            for cdep_id in state.cdeps:
                # Bundle the dependency with its hard dependency tree.
                cdep_bundle = bundle(cdep_id)
    
                # Form a new state as the union of the current state and the
                # conditional dependency bundle.
                cdep_state = state.union(cdep_bundle)
                _states.add(cdep_state)
    
                # recurse
                child_states(cdep_state)
    
        # bundle the entry.
        entry_bundle = bundle(entry_id)
        entry_state = State(entry_bundle)
    
        _states.add(entry_state)
    
        # Do it
        child_states(entry_state)
    
        return _states

    def packs(entry_id):
        _states = states(entry_id)
        _packs = {ImmutableSet(): set(_states)}
        for from_state in _states:
            for to_state in _states:
                pack = to_state.difference(from_state)
                if len(pack):
                    if from_state not in _packs:
                        _packs[from_state] = set()
                    _packs[from_state].add(pack)
        return _states, _packs
    
    pprint(packs(entry_id))

combopack(
    {
        'router': {
            'cdeps': {'index', 'user', 'userpub'},
            'deps': set()
        },
        'userpub': {
            'deps': {'user', 'listview'},
            'cdeps': set()
        },
        'user': {
            'deps': {'listview', 'index'},
            'cdeps': set()
        },
        'index': {
            'deps': set(),
            'cdeps': set()
        },
        'listview': {
            'deps': set(),
            'cdeps': set()
        }
    },
    'router'
)


combopack(
    {
        'router': {
            'cdeps': {'index', 'user'},
            'deps': set()
        },
        'user': {
            'deps': set(),
            'cdeps': set()
        },
        'index': {
            'deps': set(),
            'cdeps': set()
        }
    },
    'router'
)


#states, bundles = combopack('router')
#pprint({
#    'states': states,
#    'bundles': bundles
#})
